/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Danesh Kamal
 */

/**
 * @module nmodule/alarm/rc/db/editors/Sequencer
 * @private
 */
define(['jquery', 'lex!alarm', 'Promise',
    'bajaux/commands/Command',
    'nmodule/alarm/rc/db/editors/AlarmDetailsTable'],
  function ($, lexs, Promise, Command, AlarmDetailsTable) {

    "use strict";

    var Sequencer = function () {
    };

    Sequencer.makeFor = function (table) {

      var lex                  = lexs[0],
          index,
          model                = table.getModel(),
          selection            = table.$getSelection(),
          rows                 = model.getRows(),
          DIALOG_SELECTOR      = '.js-dialog',
          PREV_BUTTON_SELECTOR = '.js-dialog-button-prev',
          NEXT_BUTTON_SELECTOR = '.js-dialog-button-next';

      function disableButton(selector) {
        $(selector, $(DIALOG_SELECTOR)).attr('disabled', 'disabled');
      }

      function enableButton(selector) {
        $(selector, $(DIALOG_SELECTOR)).removeAttr('disabled', '');
      }

      function enableNextButton() {
        enableButton(NEXT_BUTTON_SELECTOR);
      }

      function disableNextButton() {
        disableButton(NEXT_BUTTON_SELECTOR);
      }

      function enablePreviousButton() {
        enableButton(PREV_BUTTON_SELECTOR);
      }

      function disablePreviousButton() {
        disableButton(PREV_BUTTON_SELECTOR);
      }

      function getSelectedIndex() {
        var i, index;
        for (i = 0; i < rows.length; i++) {
          if (selection.isSelected(i)) {
            index = i;
            break;
          }
        }
        return index;
      }

      function getPreviousAlarm() {

        index = index || index === 0 ? index : getSelectedIndex();

        if (index === rows.length - 1) {
          enableNextButton();
        }

        index -= 1;

        if (index === 0) {
          disablePreviousButton();
        }

        return rows[index].getSubject();
      }

      function getNextAlarm() {

        index = index || index === 0 ? index : getSelectedIndex();

        if (index === 0) {
          enablePreviousButton();
        }

        index += 1;

        if (index === rows.length - 1) {
          disableNextButton();
        }

        return rows[index].getSubject();
      }

      var PrevAlarmCommand = function () {
          },
          NextAlarmCommand = function () {
          };

      PrevAlarmCommand.make = function () {
        return {
          name: 'prev',
          displayName: lex.get('command.prevAlarm.label'),
          handler: function () {
            var prevAlarm = getPreviousAlarm(),
                tableDom  = $('.table', $('.js-dialog'));

            return Promise.resolve(prevAlarm && AlarmDetailsTable.makeFor(prevAlarm)).then(function (table) {
              if (tableDom && table) {
                tableDom.empty().append(table.jq());
              }
              return Promise.reject(); //keep the dialog open
            });
          },

          esc: false
        };
      };

      NextAlarmCommand.make = function () {
        return {
          name: 'next',
          displayName: lex.get('command.nextAlarm.label'),
          handler: function () {

            var nextAlarm = getNextAlarm(),
                tableDom  = $('.table', $('.js-dialog'));

            return Promise.resolve(nextAlarm && AlarmDetailsTable.makeFor(nextAlarm)).then(function (table) {
              if (tableDom && table) {
                tableDom.empty().append(table.jq());
              }
              return Promise.reject(); //keep the dialog open
            });
          },
          esc: false
        };
      };

      return [PrevAlarmCommand.make(), NextAlarmCommand.make()];
    };

    return Sequencer;
  });
